/*
* Copyright © 2021. TIBCO Software Inc.
* This file is subject to the license terms contained
* in the license file that is distributed with this file.
*/

var FormData = require('form-data');
var fs = require('fs');
var fsPromises = fs.promises;
var axios = require('axios');
var archiver = require('archiver');
var properties = require('properties-reader');
var debug = require('debug')('track-webapp:provisioning');

if(process.env.TEST_MODE != null && process.env.TEST_MODE.trim() === 'true')
  debug("Provisioning is using TEST MODE");

// Class to manage visualization configuration
class Provisioner {
    constructor() {
      var config = fs.readFileSync(__dirname + '/../data/provisioning/configuration.json', 
        {encoding:'utf8', flag:'r'}); 
      this.configuration = JSON.parse(config);
      this.cicProps = properties('./lib/cic.properties');
      this.testMode = process.env.TEST_MODE != null && process.env.TEST_MODE.trim() === 'true';
    }

    // Execute the provisioning process
    async execute(request, messageCallback) {
      // Register the message callback so that progress can be sent back to calling application
      this.messageCallback = messageCallback;

      // Validate that minimum required fields are present on the request
      this.sendProgressMessage("Basic validation");
      if(request.oauth == null)
        throw "OAuth key not provided";
      if(request.region == null)
        throw "Region is not specified";

      // Validate required fields for each task and build a BOM for execution 
      var tasks = this.buildBOM(request);
      
      // Iterate over each task and execute accordingly
      this.sendProgressMessage("Executing tasks");

      try {
        for(var thisTask of tasks) {
          if(thisTask.type == 'tce')
            await this.executeTCETask(thisTask);
          else if(thisTask.type == "tcds")
            await this.executeTCDSTask(thisTask);
          else
            throw "Unknown task type " + thisTask.type
        }
        this.sendProgressMessage("Completed provisioning");
        return tasks;
      }
      catch(err) {
        throw err;
      }
    }

    // Send a progress message to the message callback, if there is one
    sendProgressMessage(message) {
      debug("[Provisioning] " + message);
      if(this.messageCallback != null)
        this.messageCallback(message);
    }

    // Build a BOM for provisioning and validate all required information is available
    buildBOM(request) {
      this.sendProgressMessage("Building BOM");
      var tasks = [];

      // Iterate over the provision types ("groups")
      for(var thisType of request.provisionTypes) {
        var thisGroup = this.configuration.provisioning.types[thisType];

        // Validate the group is defined
        if(thisGroup == null)
          throw "Group " + thisType + " not found in provisioning configuration";

        // Iterate over each task in the group
        for(var thisItem of thisGroup) {
          var thisTask = this.configuration.provisioning.tasks[thisItem];

          // Validate the task is defined
          if(thisTask == null)
            throw "Task " + thisTask + " not found in provisioning configuration";

          // Append minimum required 
          thisTask.region = request.region;
          thisTask.oauth = request.oauth;
                
          // Add task to the return array
          tasks.push(thisTask);
        }
      }

      // Iterate over each of the tasks, and configure appropriately
      for(var thisTask of tasks) {
        if(thisTask.type == "tce")
          this.configureTCETask(thisTask, request);
        else if(thisTask.type == "tcds")
          this.configureTCDSTask(thisTask, request);
        else
          throw "Unknown task type " + thisTask.type;
      }

      return tasks;
    }

    // Validate the request has required fields for TCE, then set them on the task
    configureTCETask(task, request) {
      //if(request.sandboxId == null)
      //  throw "Sandbox ID not provided for TCE task";
      if(request.tcmChannelUrl == null)
        throw "TCM channel url not provided for TCE task";
      if(request.tcmChannelAuthKey == null)
        throw "TCM auth key not provided for TCE task";

      //task.sandboxId = request.sandboxId;
      task.tcmChannelUrl = request.tcmChannelUrl;
      task.tcmChannelAuthKey = request.tcmChannelAuthKey;
    }

    // Validate the request has required fields for TCDS, then set them on the task
    configureTCDSTask(task, request) {
      if(request.tcmChannelUrl == null)
        throw "TCM channel url not provided for TCDS task";
      if(request.tcmChannelAuthKey == null)
        throw "TCM auth key not provided for TCDS task";

      task.tcmChannelUrl = request.tcmChannelUrl;
      task.tcmChannelAuthKey = request.tcmChannelAuthKey;
    }

    // Execute the TCE provisioning task
    async executeTCETask(task) {
      this.sendProgressMessage("Executing TCE task for " + task.appName);

      // Create the default.cdd
      this.sendProgressMessage("Creating deployment.cdd from template");
      var templateCDDFileName = __dirname + task.location + "/" + task.cddTemplate;
      var templateCDD = fs.readFileSync(templateCDDFileName, {encoding:'utf8', flag:'r'}); 
      var defaultCDD = templateCDD
        .replace('${TCM_API_KEY}', task.tcmChannelAuthKey)
        .replace('${TCM_URL}', task.tcmChannelUrl);

        // Create a ZIP for upload
      var zip = archiver('zip');
      
      // For debugging to file
      //var output = fs.createWriteStream(__dirname + task.location + "/output.zip");
      //zip.pipe(output);

      var self = this;
      var promise = new Promise(function(resolve, reject){
        zip.on('error', function(err) {
          throw err;
        });

        zip.on('finish', async function() {
          // Create form data and append the zip as a streamed file
          var formData = new FormData();
          formData.append('files', zip, {
            filename: task.appName + ".zip",
            contentType: 'application/zip',
            knownLength: zip.pointer()
          });
          formData.append('cpu', '1');
          formData.append('noOfInstances', '1');
          formData.append('newAppName', task.appName);
          formData.append('forceReplace', 'false');

          // Push the TCE application
          await self.pushTCEApplication(task, formData);
          await self.startTCEApplication(task);
          self.sendProgressMessage("TCE task for " + task.appName + " completed");
          resolve();
        });

        // Append CDD and EAR file to zip
        zip.append(defaultCDD, {name: task.cddName});
        zip.file(__dirname + task.location + "/" + task.ear, {name: task.ear});
        zip.finalize();
      });

      return promise;
    }

    // Push TCE application to the server
    async pushTCEApplication(task, formData) {
      this.sendProgressMessage("Pushing TCE " + task.appName + ".zip to " + task.region + " region");
      if(this.testMode == true) return;

      // Compose urls
      //var url = '/tceorch/v1/sandboxes/' + encodeURI(task.sandboxId) + '/apps';
      var url = '/tceorch/v1//apps';
      var baseUrl = this.cicProps.get(task.region + '.TIBCO_CLOUD_EVENTS');

      var headers = formData.getHeaders();
      headers['Authorization'] = 'Bearer ' + task.oauth;

      // Call provisioning service
      try {      
        var response = await axios({
            url: url,
            method: 'POST',
            baseURL: baseUrl,
            headers: headers,
            data: formData
          });

        task.appId = response.data.message;
      }
      catch(err) {
        console.log(err);
        throw "Push TCE Application: " + err.response.status + " " + err.response.statusText; 
      }
    }

    // Start TCE application on the server
    async startTCEApplication(task) {
      this.sendProgressMessage("Starting TCE application " + task.appName);
      if(this.testMode == true) return;

      // TODO right now the push will start as well, but this may change in future
    }

    // Execute the TCDS task
    async executeTCDSTask(task) {
      // Retrieve streams the first time only
      if(this.streamMap == null) {
        var streams = await this.lookupStreams(task);
        this.streamMap = {};
        for(var thisStream of streams) {
            this.streamMap[thisStream.name] = thisStream;
        }
      }

      // Validate if stream already exists
      if(this.streamMap[task.streamName] != null) {
        var stream = this.streamMap[task.streamName];
        this.sendProgressMessage("TCDS stream " + task.streamName + " already exists");
        task.streamId = stream.id;
        if(stream.status == 'Stopped') {
          await this.startTCDSStream(task);
        }
        return;
      }

      this.sendProgressMessage("Executing TCDS task for " + task.streamTemplate);

      // Read the template file
      var templateFileName = __dirname + task.location + "/" + task.streamTemplate;
      var templateDefinition = fs.readFileSync(templateFileName, {encoding:'utf8', flag:'r'});

      // Update the template
      var streamDefinition = templateDefinition
        .replace('${TCM_API_KEY}', task.tcmChannelAuthKey)
        .replace('${TCM_URL}', task.tcmChannelUrl);

        try {
          await this.pushTCDSStream(task, streamDefinition);
          await this.startTCDSStream(task);
          this.sendProgressMessage("TCDS task for " + task.streamTemplate + " completed");
        }
        catch(err) {
          throw "TCDS task for " + task.streamTemplate + " failed: " + err;
        }
    }

    // Lookup streams
    async lookupStreams(task) {
      // Compose urls
      var url = '/api/v1/streams/all/runtime';
      var baseUrl = this.cicProps.get(task.region + '.TIBCO_CLOUD_DATASTREAMS');

      this.sendProgressMessage("Retrieving existing TCDS streams");
      try {
        var response = await axios({
              url: url,
              method: 'GET',
              baseURL: baseUrl,
              headers: {
                'Authorization': 'Bearer ' + task.oauth
              }
            });
        return response.data.streams;
      }
      catch(err) {
        throw "Lookup TCDS Streams: " + err.response.status + " " + err.response.statusText; 
      }
    };
  
    // Push TCDS stream definition to the server
    async pushTCDSStream(task, streamDefinition) {
      this.sendProgressMessage("Pushing TCDS " + task.streamName + " to " + task.region + " region");
      if(this.testMode == true) return;

      // Compose urls
      var url = '/api/v1/streams';
      var baseUrl = this.cicProps.get(task.region + '.TIBCO_CLOUD_DATASTREAMS');

      var headers = {};
      headers['Authorization'] = 'Bearer ' + task.oauth;
      
      // Call provisioning service 
      try {     
        var response = await axios({
            url: url,
            method: 'POST',
            baseURL: baseUrl,
            headers: headers,
            data: JSON.parse(streamDefinition)
          });

        task.streamId = response.data.id;
      }
      catch(err) {
        throw "Push TCDS Stream: " + err.response.status + " " + err.response.statusText; 
      }
    }

    // Start TCDS stream
    async startTCDSStream(task) {
      this.sendProgressMessage("Starting TCDS stream " + task.streamName);
      if(this.testMode == true) return;

      // Compose urls
      var url = '/api/v1/streams/' + task.streamId + ':start';
      var baseUrl = this.cicProps.get(task.region + '.TIBCO_CLOUD_DATASTREAMS');

      var headers = {};
      headers['Authorization'] = 'Bearer ' + task.oauth;
      
      var body = {
        "id": task.streamId
      }

      // Call start service
      try {
        await axios({
            url: url,
            method: 'POST',
            baseURL: baseUrl,
            headers: headers,
            data: body
          });
      }
      catch(err) {
        throw "Start TCDS Stream " + err.response.status + " " + err.response.statusText; 
      }
    };

}

module.exports.Provisioner = Provisioner;