/*
* Copyright © 2021. TIBCO Software Inc.
* This file is subject to the license terms contained
* in the license file that is distributed with this file.
*/

// Import required resources
var createError = require('http-errors');
var express = require('express');
var proxy = require('express-http-proxy');
var path = require('path');
var properties = require('properties-reader');
var logger = require('morgan');

// Create the application
var app = express();


// setup routers
var environmentRouter = require('./routes/environment');
var webappRouter = require('./routes/webapp');
var usersRouter = require('./routes/users');
const { resolve } = require('path');

// view engine setup
app.set('views', path.join(__dirname, 'views'));
app.set('view engine', 'jade');

app.use(logger('dev'));
app.use(express.json());
app.use(express.urlencoded({extended: false}));
app.use(express.static(path.join(__dirname, 'public')));

// Environment properties
var cicProps = properties('./lib/cic.properties');

// Setup proxy for CIC Account
app.set('trust proxy', 1);

// Proxy for login because this requires extra validation for authorization
app.use('/service/account/login', proxy(cicProps.get('us.TIBCO_CLOUD_ACCOUNT'), {
  proxyReqPathResolver: function (req) {
    return '/idm/v3/login-oauth';
  },
  proxyReqOptDecorator: function(proxyReqOpts, srcReq) {
    delete proxyReqOpts.headers['origin'];
    return proxyReqOpts;
  },
  userResHeaderDecorator(headers, userReq, userRes, proxyReq, proxyRes) {  
    var cookies = headers['set-cookie'];
    if(cookies != null) {
      for(var idx = 0; idx < cookies.length; idx++) {
        cookies[idx] = cookies[idx].replace("Domain=cloud.tibco.com;", "");
      }  
    }
    return headers;
  },
  userResDecorator: function(proxyRes, proxyResData) {
    // TODO validate authorization
    return proxyResData;
  }
}));

// Proxy for all other EU CIC account service calls
app.use('/cic/account', proxy(cicProps.get('us.TIBCO_CLOUD_ACCOUNT'), {
  proxyReqPathResolver: function (req) {
    return req.path;
  },
  proxyReqOptDecorator: function(proxyReqOpts, srcReq) {
    delete proxyReqOpts.headers['origin'];
    return proxyReqOpts;
  },
  userResHeaderDecorator(headers, userReq, userRes, proxyReq, proxyRes) {  
    var cookies = headers['set-cookie'];
    if(cookies != null) {
      for(var idx = 0; idx < cookies.length; idx++) {
        cookies[idx] = cookies[idx].replace("Domain=cloud.tibco.com;", "");
      }  
    }
    return headers;
  },
  userResDecorator: function(proxyRes, proxyResData) {
    return proxyResData;
  }
}));


// Setup environment router, no validation of cookies for these services
app.use('/', environmentRouter);

// Validate required cookies exist
function cookiesExists(req, names) {
  if(req.headers.cookie == null) return false;
  var matched = 0;
  for(var idx = 0; idx < names.length; idx++) {
    var n = names[idx] + '=';
    if(req.headers.cookie.indexOf(n) >= 0)
      matched++;
  }
  return matched == names.length;
};

// Setup validation for cookies
app.use(function(req, res, next){
  if(cookiesExists(req, ['tsc', 'domain'])) {
    next();
  }
  else {
    next(createError(404));
  }
});

// All subsequent services require cookies to be set to work properly
// Setup routers
app.use('/', webappRouter);
app.use('/users', usersRouter);

// catch 404 and forward to error handler
app.use(function(req, res, next) {
  next(createError(404));
});

// error handler
app.use(function(err, req, res, next) {
  // set locals, only providing error in development
  res.locals.message = err.message;
  res.locals.error = req.app.get('env') === 'development' ? err : {};

  // render the error page
  res.status(err.status || 500);
  res.render('error');
});

// Export the application module
module.exports = app;

// Port is being set in bin/www
//app.listen(3000);