/*
* Copyright © 2021. TIBCO Software Inc.
* This file is subject to the license terms contained
* in the license file that is distributed with this file.
*/

// Embed a Spotfire panel
app.directive('tibSpotfirePanel', function($log, $timeout) {
	return {
        template: `
            <div flex layout="column" layout-align="center center" class="spotfire-loading" ng-if="loading == true">
                <div>Loading Spotfire Dashboard... please wait</div>
                <md-progress-circular md-mode="indeterminate"></md-progress-circular>
            </div>
            <div flex layout="column" layout-align="center center" ng-if="parametersValid == false" class="spotfire-invalid">
                <div>Spotfire settings are invalid, verify Web Player URL and Analysis Path</div>
                <div class="message">{{validationMessage}}</div>
            </div>
            <div flex ng-if="parametersValid == true" id="spotfire-wrapper"></div>
            `,
		replace: false,
		transclude: true,
		restrict: 'E',
		scope: {
            configuration: '='
        },
		controller: function($scope) {
		},
		link: function(scope, element, attrs, ctrl, transclude) {
            // Initialize flags
            scope.loading = true;
            scope.parametersValid = null;

            var analysisPath = attrs.analysisPath;
            if(analysisPath == null)
                analysisPath = scope.configuration.analysisPath;
            
            // Set UI state to invalid
            var invalidUI = function(message) {
                scope.loading = false;
                scope.parametersValid = false;
                scope.validationMessage = message;
            };

            // Set UI state to valid
            var validUI = function() {
                scope.loading = false;
                scope.parametersValid = true;
            }

            // Validate configuration
            if(analysisPath == null || analysisPath.length == 0) {
                invalidUI("Analysis path is not set");
                return;
            }
            if(scope.configuration.webPlayerServerRootUrl == null || scope.configuration.webPlayerServerRootUrl.length == 0) {
                invalidUI("Web Player URL not set");
                return;
            }

            // Try to create spotfire to see if script is loaded. Expect this to throw and process in catch.
            try {
                spotfire;
                createApplication();
            }
            catch(err) {
                var scriptUrl = scope.configuration.webPlayerServerRootUrl + scope.configuration.apiUrl;
                $.getScript(scriptUrl, function(data, textStatus, xhr) {
                    if(xhr.status == 200) {
                        createApplication();
                    }
                    else {
                        invalidUI();
                    }
                }).fail(function(err) {
                    if(err.status == 404)                  
                        invalidUI("Script " + scriptUrl + " not found");
                    else
                        invalidUI("Script loading error: " + err.statusText);
                });
            }
            
            // Create a new spotfire application
            function createApplication() {
                spotfire.webPlayer.createApplication(
                    scope.configuration.webPlayerServerRootUrl,
                    scope.configuration.customizationInfo,
                    analysisPath,
                    scope.configuration.parameters,
                    scope.configuration.reloadInstances,
                    scope.configuration.apiVersion,
                    onReadyCallback,
                    onCreateLoginElement 
                );
            }

            // Callback to execute once ready
            function onReadyCallback(response, newApp) {
                if(response == null)
                    return;

                if(response.status === "OK" && scope.configuration.onReady != null)
                    scope.configuration.onReady();

                app = newApp;
                if(response.status === "OK") {
                    validUI();
                    scope.$digest();

                    var id = Math.random() * 16 + '-container';
                    var containerElem = document.createElement('div');
                    containerElem.id = id;

                    var wrapperElem = element.find("#spotfire-wrapper");
                    wrapperElem.append(containerElem);

                    doc = app.openDocument(id, 0);
                }
                else {
                    invalidUI(response.message);
                    console.log("Status not OK. " + response.status + ": " + response.message)
                }
            }
         
            // Callback to execute when error
            function onError(error) {
                console.error("Spotfire Error: " + error);
                invalidUI(error);
                scope.$digest();
            }
         
            // Callback to execute when login element is created
            function onCreateLoginElement() {
                if(scope.configuration.onCreateLoginElement != null)
                    scope.configuration.onCreateLoginElement();
            }
   
        }
	};	
});
