/*
* Copyright © 2021. TIBCO Software Inc.
* This file is subject to the license terms contained
* in the license file that is distributed with this file.
*/

var express = require('express');
var router = express.Router();
var fs = require('fs');
var axios = require('axios');
var properties = require('properties-reader');
var Simulator = require('../lib/simulator')
var spotfireDXP = require('../lib/spotfire_dxp');
const { route } = require('./users');

// Environment properties
var cicProps = properties('./lib/cic.properties');

// Start page for redirects
var START_PAGE = 'app/tracking/index.html';

// Simulator singletom
var simulator = null;

// --------------------------------------------------------------------------------
// ROOT
router.get('/', function(req, res, next) {
  res.redirect(301, START_PAGE);
  //res.render('index', { title: 'Express' });
});

// --------------------------------------------------------------------------------
// CONTEXT
// Read the configuration file
router.get('/service/context/configuration', function(req, res, next) {
  fs.readFile(__dirname + '/../data/context/configuration.json', function (err, data) {
    if (err) {
      throw err; 
    }
    let response = JSON.parse(data.toString());
    res.json(response);
  });
});

// Write the configuration file
router.post('/service/context/configuration', function(req, res, next) {
  var payload = JSON.stringify(req.body, null, 2);
  fs.writeFile( __dirname + '/../data/context/configuration.json', payload, function (err, data) {
    if (err) {
      throw err; 
    }
    res.json("{}");
  });
});


// --------------------------------------------------------------------------------
// OAUTH
// Retrieve TCM channels
router.get('/service/cic/oauth/datastreams/channels/:region', function(req, res, next) {
  var region = req.params.region;
  if(region == null) {
    res.status(500).send("Invalid region");
    return;
  }

  var url = '/ui/api/v1/plugins/tcm/channels';
  var baseUrl = cicProps.get(region + '.TIBCO_CLOUD_DATASTREAMS');

  axios({
    url: url,
    method: 'GET',
    baseURL: baseUrl,
    headers: {
      'Authorization': req.headers['authorization']
    }
  })
  .then(function(response){
    res.status(200).send(response.data);
  })
  .catch(function(err){
    res.status(err.response.status).send(err);
  });

});

// Retrieve TCE sandboxes
router.get('/service/cic/oauth/events/sandboxes/:region', function(req, res, next) {
  var region = req.params.region;
  if(region == null) {
    res.status(500).send("Invalid region");
    return;
  }

  var url = '/tceorch/v1/sandboxes';
  var baseUrl = cicProps.get(region + '.TIBCO_CLOUD_EVENTS');

  axios({
    url: url,
    method: 'GET',
    baseURL: baseUrl,
    headers: {
      'Authorization': req.headers['authorization']
    }
  })
  .then(function(response){
    res.status(200).send(response.data);
  })
  .catch(function(err){
    res.status(err.response.status).send(err);
  });

});

// Download DXP
router.get('/service/spotfire/analysis/:region/:oauth', async function(req, res, next) {
  var region = req.params.region;
  var oauth = "Bearer " + req.params.oauth;
  spotfireDXP.download(region, oauth, res);
});


// --------------------------------------------------------------------------------
// SIMULATION
// Get testcases
router.get('/service/simulation/testcases', function(req, res, next) {
  const testFolder = __dirname + '/../data/testcases';
  fs.readdir(testFolder, function(err, files) {
    let testcases = [];
    files.forEach(function(file) {
      let testcase = JSON.parse(fs.readFileSync(testFolder + "/" + file).toString());
      testcases.push(testcase);
    });

    let response = {
      "testcases": testcases
    }
    res.json(response);
  });
});  

// Start testcase
router.post('/service/simulation/testcase/start', async function(req, res, next) {
  // If simulator is not yet initialized, create it now
  if(simulator == null) {
    // Validate TCM configuration
    var configuration = JSON.parse(fs.readFileSync( __dirname + '/../data/context/configuration.json'));
    var tcmChannel = configuration.messaging;
    if(tcmChannel.url == null || tcmChannel.authKey == null) {
      res.status(500).send("TCM channel not defined");
      return;
    }

    // Create and connect the simulator to eFTL
    try {
      simulator = new Simulator.Simulator(tcmChannel);
      await simulator.connect();
    }
    catch(err) {
      res.status(500).send(err);
      return;
    }
  }

  // Start a new simulation
  try {
    var barcode = await simulator.simulate(req.body.testcase, req.body.compression);
    var response = {
      barcode: barcode,
    }
    res.status(200).send(JSON.stringify(response));
    return;
  }
  catch(err) {
    console.log(err);
    res.status(500).send(err);
  }
});

module.exports = router;
